// Cake Zombies

#ifndef CAKE_ZOM_H
#define CAKE_ZOM_H

// INCLUDES ///////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN  

// you must #define INITGUID if not done elsewhere
#define INITGUID

#include <windows.h>   // include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <objbase.h>
#include <iostream> // include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>
#include <SYS\Stat.h>


#include <ddraw.h>     // directX includes
#include <dinput.h>
#include <dsound.h> // include dsound, dmusic
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include "T3DLIB1.H"
#include "T3DLIB3.H"

#define ROWBYTES_DWORD(width,bitcount)   ((((width)*(bitcount)+31) >> 3) & 0xfffc) 

#define ANIM_RIGHT		0
#define ANIM_LEFT		1

#define LEVEL1 "level2.txt"
#define LEVEL2 "level3.txt"

// game mode
#define MENU			0
#define GAME			1
#define WIN				2

// sounds
#define ENEMY_GUN		0
#define SHOTGUN			1
#define HURT			2
#define DIE				3
#define ZOM_DMG			4
#define ZOM_DIE			5

#define CRAWL_SPEED		1
#define WALK_SPEED		3
#define RUN_SPEED		5
#define WAIT_SPEED		200
#define GROUND_ACC		0.2
#define AIR_ACC			0.1

#define GRAVITY			0.2
#define JUMP_HEIGHT		170
#define JUMP_VEL		5
#define JUMP_FRAME(dir)	3+dir*4
#define LAND_FRAME(dir) 1+dir*4
#define STND_FRAME(dir) 0+dir*5
#define AGRO_FRAME(dir) 3+dir*5
#define DEAD_FRAME(dir) 4+dir*5

#define MAX_BULLETS		500
#define BULLET_VEL		3

// All object types
#define OBJ_START		12
#define OBJ_ZOMBIE		13
#define OBJ_DRONE		14
#define OBJ_FINISH		15

// BOB.varF[INDEX]
#define FORCE_X			0
#define FORCE_Y			1
#define COF				2

// BOB.varI[INDEX]
#define OBJ_TYPE		0
#define OBJ_WORLD_X		1
#define OBJ_WORLD_Y		2
#define AIR				3
#define NPC_BEHAV		4
#define NPC_DIR			5
#define NPC_DUR			6
#define BOB_X_NEAR		7
#define BOB_X_FAR		8
#define ZOM_LIFE		9
#define DEATH_ANIM		10

// collision room for player
#define P_FEET_ROOM		2
#define P_ELBOW_ROOM	20

// collision room for zombie
#define Z_FEET_ROOM		2
#define Z_ELBOW_ROOM	24
	
// animation sequences for bob
const int zom_anims[2][4] = { {0,1,0,2},
						   {0+5,1+5,0+5,2+5},};

const int ply_anims[2][4] = { {0,1,0,2},
						   {0+4,1+4,0+4,2+4},};

// zombie behaviors
#define ZOM_STAND		0
#define ZOM_WALK		1
#define ZOM_AGGRO		2
#define ZOM_SHOOT		3
#define ZOM_WALK_SPD	1
#define ZOM_HEALTH		100
#define MIN_DUR			20
#define MAX_DUR			70
#define AGGRO_RANGE		200
#define WEAPON_COOL		80
#define WEAPON_FLASH	5


#define PLAYER_COOL		80
#define PLAYER_POW		150


// how far left or right will player go before
// camera starts scrolling through level
#define SCROLL_LEFT		213
#define SCROLL_RIGHT	280

// default screen size
#define SCREEN_WIDTH    640  // size of screen
#define SCREEN_HEIGHT   480
#define SCREEN_BPP      8    // bits per pixel

#define BITMAP_ID            0x4D42 // universal id for a bitmap
#define MAX_COLORS_PALETTE   256

#define NUM_TEXTURES         12 // normally this would be a multiple of 16 or something!
#define NUM_ZOMBIE_FRAMES    12 // normally this would be a multiple of 16 or something!

// PROTOTYPES /////////////////////////////////////////////

// game console
int Game_Init(void *parms=NULL, int num_parms = 0);
int Game_Shutdown(void *parms=NULL, int num_parms = 0);
int Main_Menu();
int Win_Stage();
int Game_Main(void *parms=NULL,  int num_parms = 0);
int Player_Input(BOB*);
int Prep_File(char*);
int Load_Level(char*);
int Unload_Level();
int Save_Level(char*);
int Expand_Level();
int Add_Object(int, int, int);
int Update_Camera(BOB*);
int Update_BOB(BOB*);
int Y_Collision(BOB*);
int X_Collision(BOB*);
int Zombie_Behavior(BOB*);
int Add_Bullet(BOB*, BOB*);
int Draw_Bullets();
int Calc_Bullets();
int Take_Dmg(BOB*);
int Revive_Monsters();
int Fire_Weapon();

// GLOBALS ////////////////////////////////////////////////

// windows vars
HWND      main_window_handle = NULL; // globally track main window
int       window_closed      = 0;    // tracks if window is closed
HINSTANCE main_instance      = NULL; // globally track hinstance

// INPUT STUFF ///////////////////////////////////////////
LPDIRECTINPUT8        lpdi      = NULL;    // dinput object
LPDIRECTINPUTDEVICE8  lpdikey   = NULL;    // dinput keyboard
LPDIRECTINPUTDEVICE8  lpdimouse = NULL;    // dinput mouse
LPDIRECTINPUTDEVICE8  lpdijoy   = NULL;    // dinput joystick 
GUID                 joystickGUID;        // guid for main joystick
char                 joyname[80];         // name of joystick
char				 mapname[80];

// these contain the target records for all di input packets
UCHAR        keyboard_state[256]; // contains keyboard state table
DIMOUSESTATE mouse_state;         // contains state of mouse
DIJOYSTATE   joy_state;           // contains state of joystick

int mouse_x,                    // used to track mouse
    mouse_y;

float var_jump = 0;
int jump_key = 0;
    
UCHAR mouse_color=100;          // color of mouse brush

int command_state=0;            // state of user command

// position of control buttons
int buttons_x[] = {509, 559, 509, 559};
int buttons_y[] = {344, 344, 383, 383};

// on/off state of buttons
int buttons_state[] = {0,1,0,0};

char buffer[80];                // used to print text

int sound_ids[6]; // storage for 4 sound fx ids

// demo globals
BOB textures;     // texture memory
//BOB zombie;	 
BOB *Obj_List;

typedef struct
{
	int active;
	float slope;
	float world_x;
	float world_y;
	float screen_x;
	float screen_y;
	float vel_x;
	float vel_y;
} BULLETS, *PBULLETS;

BULLETS Bullet_List[MAX_BULLETS];

int current_bullet = 0;

int PLAYER_BOB;

int world_x = 0,   // current position of viewing window
    world_y = 0;

int mouse_map_x = 0,
	mouse_map_y = 0; // world mouse positions

int mouse_drag_x_start = 0,
	mouse_drag_y_start = 0,
	mouse_drag_x_end = 0,
	mouse_drag_y_end = 0;

int drag_brush = 0;

int Level_Size;
int *Level[7];

int Num_Objects = 0;

int Player_Life = 100;
int Player_Lives = 3;


int checkpoint_x = 0;
int checkpoint_y = 0;

int GAME_MODE = 0;

int temp_dmg = 0;

int current_level = 2;
int win_pos = 0;


char Next_Level[30];


#endif



